{#
.. screenshot:: |canaille|/reset
   :align: right
   :width: 275px

   The password reset page.

The password reset template.

Displays a password reset form.

:param form: The password reset form.
:type form: :class:`~canaille.core.endpoints.forms.PasswordResetForm`
:param user: The user associated with the URL.
:type user: :class:`~canaille.core.models.User`
:param hash: The secret link hash.
:type hash: :class:`str`
#}
{% extends theme('base.html') %}
{% import 'macro/form.html' as fui %}

{% block content %}
    <div class="loginform">
        {% block header %}
            <h3 class="ui top attached header">
                {% if logo_url %}
                    <img class="ui image" src="{{ logo_url }}">
                {% endif %}
                <div class="content">
                    {% trans %}Password reset{% endtrans %}
                </div>
            </h3>
        {% endblock %}

        <div class="ui attached clearing segment">
            {{ fui.render_form(form, _("Password reset")) }}
        </div>

    </div>
{% endblock %}
