{#
.. screenshot:: |canaille|/about
   :align: right
   :context: user
   :width: 275px

   The 'About' page.

The 'About' page.

This is an informational page, displaying the project links.

:param version: The current Canaille version.
:type version: :class:`str`

#}

{% extends theme('base.html') %}
{% import 'macro/form.html' as fui %}

{%- block title -%}
    {%- trans -%}About Canaille{%- endtrans -%}
{%- endblock -%}

{% block content %}
    <div class="ui clearing segment">
        <a href="{{ url_for('core.account.index') }}">
            <img class="ui tiny centered image" src="{{ url_for("static", filename="img/canaille-head.webp") }}" alt="{{ website_name }}">
        </a>

        <h2 class="ui center aligned header">
            <div class="content">
                {{ _("About Canaille") }}
            </div>
            <div class="sub header">{% trans %}Free and open-source identity provider.{% endtrans %}</div>
        </h2>
        <p style="text-align: center">
            <a href="https://pypi.org/project/canaille/">{% trans %}Version {{ version }}{% endtrans %}</a> ·
            <a href="{{ documentation_url }}">{% trans %}Documentation{% endtrans %}</a> ·
            <a href="https://gitlab.com/yaal/canaille">{% trans %}Source code{% endtrans %}</a>
        </p>

    </div>
{% endblock %}
