{# Authorization details template.

Displays details about an :class:`~canaille.oidc.basemodels.AuthorizationCode`.

:param authorization: The detailed authorization.
:type authorization: :class:`~canaille.oidc.basemodels.AuthorizationCode`.
#}
{% extends theme('base.html') %}

{%- block title -%}
    {%- trans %}View an authorization{% endtrans -%}
{%- endblock -%}

{% block submenu %}
    <a class="item" href="{{ url_for('core.admin.mail_index') }}">
        <i class="user mail icon"></i>
        {% trans %}Emails{% endtrans %}
    </a>
    <a class="item" href="{{ url_for('oidc.clients.index') }}">
        <i class="th list icon"></i>
        {% trans %}Clients{% endtrans %}
    </a>
    <a class="item" href="{{ url_for('oidc.clients.add') }}">
        <i class="plus icon"></i>
        {% trans %}Add a client{% endtrans %}
    </a>
    <a class="item" href="{{ url_for('oidc.tokens.index') }}">
        <i class="key icon"></i>
        {% trans %}Tokens{% endtrans %}
    </a>
    <a class="active item" href="{{ url_for('oidc.authorizations.index') }}">
        <i class="user secret icon"></i>
        {% trans %}Codes{% endtrans %}
    </a>
{% endblock %}

{% block content %}
    <div class="loginform">
        <h3 class="ui top attached header">
            {% trans %}View an authorization{% endtrans %}
        </h3>

        <div class="ui attached clearing segment">
            <ul>
                {% for attr in authorization.attributes %}
                    <li>{{ attr }}: {{ authorization[attr]|string }}</li>
                {% endfor %}
            </ul>
        </div>
    </div>
{% endblock %}
